unit rttitojson.writer;

{$mode ObjFPC}
{$H+}
{$modeswitch advancedrecords}

interface

uses
  Classes, SysUtils, fpJSON, typinfo, rtti, rttitojson.types;

Type

  { TRTTIJSONWriter }

  TRTTIJSONWriter = record
  private
    Ctx : TRTTIContext;
    function DateFieldToJSON(aDate: TDateTime; Fld: TRTTIField): TJSONData;
    function FieldToJSON(aData: Pointer; Fld: TRTTIField): TJSONData;
  Public
    Procedure ToJSON(aData : Pointer; aTypeInfo : PTypeInfo; aJSON : TJSONObject);
    Generic function ToJSON<T>(var aData : T) : TJSONObject;
    class function create : TRTTIJSONWriter; static;
    procedure Free;
  end;

implementation

uses dateutils;

{ TRTTIJSONWriter }

function TRTTIJSONWriter.DateFieldToJSON(aDate : TDateTime; Fld : TRTTIField) : TJSONData;

var
  Res,Fmt : string;
  DateFormat : DateFormatAttribute;

begin
  Fmt:='';
  DateFormat:=DateFormatAttribute(Fld.GetAttribute(DateFormatAttribute));
  if Assigned(DateFormat) then
    Fmt:=DateFormat.DateFormat;
  if Fmt='' then
    Res:=DateToISO8601(aDate)
  else
    Res:=FormatDateTime(Fmt,aDate);
  Result:=TJSONString.Create(Res);
end;

function TRTTIJSONWriter.FieldToJSON(aData: Pointer; Fld : TRTTIField): TJSONData;

var
  V : TValue;
begin
  V:=Fld.GetValue(aData);
  case V.Kind of
    tkFloat : if V.TypeInfo=TypeInfo(TDateTime) then
                Result:=DateFieldToJSON(V.AsDateTime,Fld)
              else
                Result:=TJSONFloatNumber.Create(V.AsDouble);
    tkInt64 : Result:=TJSONInt64Number.Create(V.AsInt64);
    tkInteger : Result:=TJSONIntegerNumber.Create(V.AsInteger);
    tkEnumeration : Result:=TJSONString.Create(GetEnumName(V.TypeInfo,V.AsInteger));
    tkAString,
    tkString : Result:=TJSONString.Create(V.AsString);
    tkWString,
    tkUString : Result:=TJSONString.Create(UTF8Encode(V.AsUnicodeString));
  else
    Raise Exception.Create('Unsupported type');
  end;
end;

procedure TRTTIJSONWriter.ToJSON(aData: Pointer; aTypeInfo: PTypeInfo; aJSON: TJSONObject);

Var
  R : TRttiRecordType;
  Fld : TRttiField;
  Key : JSONKeyAttribute;
  KeyName : String;

begin
  R:=Ctx.GetType(aTypeInfo) as TRttiRecordType;
  For Fld in R.GetFields do
    begin
    KeyName:=Fld.Name;
    Key:=JSONKeyAttribute(Fld.GetAttribute(JSONKeyAttribute));
    if Assigned(Key) then
      KeyName:=Key.Key;
    aJSON.Add(KeyName,FieldToJSON(aData,Fld));
    end;
end;

generic function TRTTIJSONWriter.ToJSON<T>(var aData : T) : TJSONObject;

begin
  Result:=TJSONObject.Create;
  try
    ToJSON(@aData,TypeInfo(T),Result);
  except
    Result.Free;
    Raise;
  end;
end;

class function TRTTIJSONWriter.create: TRTTIJSONWriter;

begin
  Result.Ctx:=TRttiContext.Create(False);
end;

procedure TRTTIJSONWriter.Free;
begin
  Ctx.Free;
end;

end.

